# -*- coding: utf8 -*-
"""Scraper class used to retrieve Whyd source links and download audio tracks.
"""
from __future__ import (absolute_import, division,
                        print_function, unicode_literals)
from builtins import *

import logging
import os

from bs4 import BeautifulSoup
from youtube_dl import YoutubeDL
from youtube_dl.utils import DownloadError
import requests

# Default output format
YDL_PARAMS = {
    'outtmpl': '%(title)s.%(ext)s',
    'nooverwrites': True,
    'postprocessors': [{
        'key': 'FFmpegExtractAudio',
        'preferredcodec': 'mp3',
        'preferredquality': '256'}]
}

# Whyd root URL
ROOT_URL = 'https://whyd.com'

# Supported providers
PROVIDERS = {
    'soundcloud': {
        'prefix': '/sc/',
        'root_url': 'https://soundcloud.com/'},
    'youtube': {
        'prefix': '/yt/',
        'root_url': 'https://youtu.be/'},
    'vimeo': {
        'prefix': '/vi/',
        'root_url': 'https://vimeo.com/'}
}

# Max. tracks to scrap
PLAYLIST_LIMIT = '300'


class WhydScraper():

    """ All your tracks are belong to us. """

    def __init__(self):
        self.ydl = YoutubeDL(YDL_PARAMS)
        self.ydl.add_default_info_extractors()

    def _make_soup(self, url):
        """ Helper to make a BeautifulSoup object.

            :param url: URL to scrap.
        """
        try:
            return BeautifulSoup(requests.get(url).text)
        except ValueError:
            logging.warning('The URL you provided is not valid : "%s"' % url)
            return None

    def scrap_playlists(self, username):
        """ Scrap user playlists from his profile page.

            :param username: User name (slug) OR id (/u/xxxx).
        """
        soup = self._make_soup('{root_url}/{username}/playlists'.format(
            root_url=ROOT_URL, username=username))

        pl_tags = soup.find_all('li', class_='playlist')

        return [li.a['href'].split('/')[-1] for li in pl_tags]

    def get_playlist_by_id(self, username, pl_id, limit=PLAYLIST_LIMIT):
        """ Get tracklist by Username + ID."""
        res = requests.get(
            '{root_url}/{username}/playlist/{pl_id}?format=json&limit={limit}'.format(
                root_url=ROOT_URL, username=username, pl_id=pl_id, limit=limit))

        tracks = self._format_json(res.json())
        return tracks

    def get_playlist_by_url(self, pl_url, limit=PLAYLIST_LIMIT):
        """ Get tracklist by URL. """
        res = requests.get('{url}?format=json&limit={limit}'.format(
            url=pl_url, limit=limit))
        tracks = self._format_json(res.json())
        return tracks

    def _format_json(self, json):
        """ Parse JSON and return formated dict. """
        # format urls
        for track in json:
            for e in PROVIDERS:
                if PROVIDERS[e]['prefix'] in track['eId']:
                    track['eId'] = track['eId'].replace(
                        PROVIDERS[e]['prefix'],
                        PROVIDERS[e]['root_url']).split('#')[0]

            try:
                playlist = track['pl']['name']
            except KeyError:
                playlist = 'None'
        return [{'playlist': playlist,
                 'name': track['name'],
                 'url': track['eId']} for track in json]

    def get_user_stream(self, username, limit='20'):
        """ Get user tracklist from its added tracks stream.

            :param username: User nickname (slug) OR id (/u/xxxx).
            :param limit: Maximum number of tracks to retrieve.
        """
        res = requests.get(
            '{root_url}/{username}?format=json&limit={limit}'.format(
                root_url=ROOT_URL, username=username, limit=limit))

        tracks = self._format_json(res.json())
        return tracks

    def get_user_likes(self, username):
        """ Get user tracklist from its liked tracks stream.

            :param username: User nickname (slug) OR id (/u/xxxx).
        """
        res = requests.get(
            '{root_url}/{username}/likes?format=json'.format(
                root_url=ROOT_URL, username=username))

        tracks = self._format_json(res.json())
        return tracks

    def download(self, url, outdir, dry_run=False):
        """ Download the audio version of a given track url.

            :param url: Track URL.
            :param outdir: Output directory.
        """
        if dry_run:
            print(url)
            return
        # output root folder
        try:
            os.mkdir('output')
        except OSError:
            pass
        # playlist specific folder
        try:
            os.mkdir('output/%s' % outdir)
            logging.info('output/%s created.' % outdir)
        except OSError:
            logging.info('output directory already exists, skipping creation.')

        self.ydl.params['outtmpl'] = 'output/' + outdir + '/%(title)s.%(ext)s'
        try:
            self.ydl.extract_info(url, download=True)
        except DownloadError as e:
            logging.info(e)
            return



# coding: utf8
from __future__ import (absolute_import, division,
                        print_function, unicode_literals)
from builtins import *

import logging

import click
from slugify import slugify
from whydtogo.scraper import WhydScraper


__author__ = 'Julien Tanay'
__version__ = '0.4.1'
VERSION = __version__

@click.group()
def main():
    pass

@main.command()
@click.argument('username')
@click.option('--likes', default=False, help='Fetch user likes (last tracks)')
@click.option('--stream', default=False, help='Fetch user stream (last tracks)')
@click.option('--total', default=False, help='Fetch all user tracks (caution!)')
@click.option('--dryrun', default=False, help='Only print the result')
def user(username, likes, stream, total, dryrun):
    ws = WhydScraper()
    if likes:
        tracklist = ws.get_user_likes(username)
        for track in tracklist:
            ws.download(
                track['url'],
                outdir=slugify('{user}_likes'.format(
                    user=username)),
                dry_run=dryrun)

    elif stream:
        tracklist = ws.get_user_stream(username)
        for track in tracklist:
            ws.download(
                track['url'],
                outdir=slugify(username),
                dry_run=dryrun)

    elif total:
        tracklist = ws.get_user_stream(username, limit='3000')
        for track in tracklist:
                ws.download(
                    track['url'],
                    outdir=slugify(username),
                    dry_run=dryrun)

    else:
        for playlist_id in ws.scrap_playlists(username):
            playlist = ws.get_playlist_by_id(
                username, playlist_id)

            for track in playlist:
                    ws.download(
                        track['url'],
                        outdir=slugify(track['playlist']),
                        dry_run=dryrun)
    return


@main.command()
@click.argument('urls', nargs=-1)
@click.option('--dryrun', help='Only print the result')
def playlist(url, print):
    ws = WhydScraper()
    for url in urls:
        playlist = ws.get_playlist_by_url(url)

        for track in playlist:
            ws.download(
                track['url'],
                outdir=slugify(track['playlist']),
                dry_run=dryrun)
    return

if __name__ == '__main__':
    main()



import codecs
import os.path
import re

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


def fpath(name):
    return os.path.join(os.path.dirname(__file__), name)


def read(fname):
    return codecs.open(fpath(fname), encoding='utf-8').read()


def grep(attrname):
    pattern = r"{0}\W*=\W*'([^']+)'".format(attrname)
    strval, = re.findall(pattern, file_text)
    return strval

file_text = read(fpath('whydtogo/__init__.py'))

setup(
    name='WhydToGo',
    version=grep('__version__'),
    description='Whyd To Go - Take your Whyd playlists away',
    long_description=read(fpath('README.rst')),
    url='https://github.com/Djiit/whydtogo',
    author='Julien Tanay',
    author_email='julien.tanay@gmail.com',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    keywords='whyd scraping youtube-dl',
    packages=['whydtogo'],
    install_requires=read(fpath('requirements.txt')).splitlines(),
    test_suite="tests",
    entry_points={
        'console_scripts': [
            'whydtogo=whydtogo:main',
        ],
    },
)
